use std::io::Write;

pub use cliask_derive::ActionEnum;

pub enum AskError {
    IOError(std::io::Error)
}

impl From<std::io::Error> for AskError {
    fn from(value: std::io::Error) -> Self {
        Self::IOError(value)
    }
}

pub trait ActionEnum: 'static {
    const ACTION_COUNT : usize;
    const LABELS : &'static [&'static str];
    const KEYS : &'static [char];
    fn try_parse(from: char) -> Option<Self> where Self: Sized;
}

pub struct ActionPrompt<'l, AE: ActionEnum> {
    default_action: Option<AE>,
    _ghost: std::marker::PhantomData<&'l AE>
}

impl<'l, AE: ActionEnum> ActionPrompt<'l, AE> {
    pub fn new() -> Self {
        Self { default_action: None, _ghost: std::marker::PhantomData }
    }
    pub fn run(self) -> Result<AE, AskError> {
        let mut stdout = console::Term::stdout();

        let mut options = String::new();
        let mut first = true;
        for (label,key) in AE::LABELS.iter().zip(AE::KEYS) {
            let key_string = key.to_string();
            let emphasized_label = label.replace(&key_string, format!("({})", console::style(&key_string).underlined()).as_str());
            if first {
                first = false
            } else {
                options.push('/');
            }
            options += format!("{}", emphasized_label).as_str();
        }

        options.push('?');

        stdout.write(options.as_bytes())?;

        loop {
            if let Some(action) = AE::try_parse(stdout.read_char()?) {
                return Ok(action)
            }
        }
    }
}

pub struct SelectPrompt<'l> {
    prompt: &'l str,
}


